"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.keyboardAction = keyboardAction;
exports.releaseAllKeys = releaseAllKeys;

var _dom = require("@testing-library/dom");

var _utils = require("../utils");

var plugins = _interopRequireWildcard(require("./plugins"));

var _getEventProps = require("./getEventProps");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

async function keyboardAction(config, actions) {
  for (let i = 0; i < actions.length; i++) {
    await keyboardKeyAction(config, actions[i]);

    if (typeof config.delay === 'number' && i < actions.length - 1) {
      await (0, _utils.wait)(config.delay);
    }
  }
}

async function keyboardKeyAction(config, {
  keyDef,
  releasePrevious,
  releaseSelf,
  repeat
}) {
  const {
    document,
    keyboardState,
    delay
  } = config;

  const getCurrentElement = () => getActive(document); // Release the key automatically if it was pressed before.


  const pressed = keyboardState.pressed.find(p => p.keyDef === keyDef);

  if (pressed) {
    await keyup(keyDef, getCurrentElement, config, pressed.unpreventedDefault);
  }

  if (!releasePrevious) {
    let unpreventedDefault = true;

    for (let i = 1; i <= repeat; i++) {
      unpreventedDefault = await keydown(keyDef, getCurrentElement, config);

      if (unpreventedDefault && hasKeyPress(keyDef, config)) {
        await keypress(keyDef, getCurrentElement, config);
      }

      if (typeof delay === 'number' && i < repeat) {
        await (0, _utils.wait)(delay);
      }
    } // Release the key only on the last iteration on `state.repeatKey`.


    if (releaseSelf) {
      await keyup(keyDef, getCurrentElement, config, unpreventedDefault);
    }
  }
}

function getActive(document) {
  var _getActiveElement;

  return (_getActiveElement = (0, _utils.getActiveElement)(document)) != null ? _getActiveElement :
  /* istanbul ignore next */
  document.body;
}

async function releaseAllKeys(config) {
  const getCurrentElement = () => getActive(config.document);

  for (const k of config.keyboardState.pressed) {
    await keyup(k.keyDef, getCurrentElement, config, k.unpreventedDefault);
  }
}

async function keydown(keyDef, getCurrentElement, config) {
  const element = getCurrentElement(); // clear carried characters when focus is moved

  if (element !== config.keyboardState.activeElement) {
    config.keyboardState.carryValue = undefined;
    config.keyboardState.carryChar = '';
  }

  config.keyboardState.activeElement = element;
  applyPlugins(plugins.preKeydownBehavior, keyDef, element, config);

  const unpreventedDefault = _dom.fireEvent.keyDown(element, (0, _getEventProps.getKeyEventProps)(keyDef, config.keyboardState));

  config.keyboardState.pressed.push({
    keyDef,
    unpreventedDefault
  });

  if (unpreventedDefault) {
    // all default behavior like keypress/submit etc is applied to the currentElement
    applyPlugins(plugins.keydownBehavior, keyDef, getCurrentElement(), config);
  }

  return unpreventedDefault;
}

async function keypress(keyDef, getCurrentElement, config) {
  const element = getCurrentElement();

  const unpreventedDefault = _dom.fireEvent.keyPress(element, { ...(0, _getEventProps.getKeyEventProps)(keyDef, config.keyboardState),
    charCode: keyDef.key === 'Enter' ? 13 : String(keyDef.key).charCodeAt(0)
  });

  if (unpreventedDefault) {
    applyPlugins(plugins.keypressBehavior, keyDef, getCurrentElement(), config);
  }
}

async function keyup(keyDef, getCurrentElement, config, unprevented) {
  const element = getCurrentElement();
  applyPlugins(plugins.preKeyupBehavior, keyDef, element, config);

  const unpreventedDefault = _dom.fireEvent.keyUp(element, (0, _getEventProps.getKeyEventProps)(keyDef, config.keyboardState));

  if (unprevented && unpreventedDefault) {
    applyPlugins(plugins.keyupBehavior, keyDef, getCurrentElement(), config);
  }

  config.keyboardState.pressed = config.keyboardState.pressed.filter(k => k.keyDef !== keyDef);
  applyPlugins(plugins.postKeyupBehavior, keyDef, element, config);
}

function applyPlugins(pluginCollection, keyDef, element, config) {
  const plugin = pluginCollection.find(p => p.matches(keyDef, element, config));

  if (plugin) {
    plugin.handle(keyDef, element, config);
  }

  return !!plugin;
}

function hasKeyPress(keyDef, config) {
  var _keyDef$key;

  return (((_keyDef$key = keyDef.key) == null ? void 0 : _keyDef$key.length) === 1 || keyDef.key === 'Enter') && !config.keyboardState.modifiers.ctrl && !config.keyboardState.modifiers.alt;
}