"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setupDirect = setupDirect;
exports.setupMain = setupMain;
exports.setupSub = setupSub;

var _document = require("../document");

var _keyboard = require("../keyboard");

var _pointer = require("../pointer");

var _options = require("../options");

var _utils = require("../utils");

var _config = require("./config");

var userEventApi = _interopRequireWildcard(require("./api"));

var _wrapAsync = require("./wrapAsync");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Start a "session" with userEvent.
 * All APIs returned by this function share an input device state and a default configuration.
 */
function setupMain(options = {}) {
  var _doc$defaultView;

  const doc = getDocument(options);
  (0, _document.prepareDocument)(doc);
  const view = (_doc$defaultView = doc.defaultView) != null ? _doc$defaultView :
  /* istanbul ignore next */
  window;
  (0, _utils.attachClipboardStubToView)(view);
  return doSetup({ ..._options.defaultOptionsSetup,
    keyboardState: (0, _keyboard.createKeyboardState)(),
    pointerState: (0, _pointer.createPointerState)(doc)
  });
}
/**
 * Setup in direct call per `userEvent.anyApi()`
 */


function setupDirect(options = {}, node) {
  const doc = getDocument(options, node);
  (0, _document.prepareDocument)(doc);
  return doSetup({
    keyboardState: (0, _keyboard.createKeyboardState)(),
    pointerState: (0, _pointer.createPointerState)(doc),
    ..._options.defaultOptionsDirect,
    ...options
  });
}
/**
 * Create a set of callbacks with different default settings but the same state.
 */


function setupSub(options) {
  return doSetup({ ...this[_config.Config],
    ...options
  });
}

function wrapImpl(impl) {
  function method(...args) {
    return (0, _wrapAsync.wrapAsync)(() => impl.apply(this, args));
  }

  Object.defineProperty(method, 'name', {
    get: () => impl.name
  });
  return method;
}

const wrappedApis = Object.fromEntries(Object.entries(userEventApi).map(([name, impl]) => [name, wrapImpl(impl)]));

function doSetup(config) {
  return { ...wrappedApis,
    setup: setupSub,
    [_config.Config]: config
  };
}

function getDocument(options, node) {
  var _ref, _options$document;

  return (_ref = (_options$document = options.document) != null ? _options$document : node && (0, _utils.getDocumentFromNode)(node)) != null ? _ref : document;
}